import { highlightSpecialChars, drawSelection, highlightActiveLine, keymap, ViewPlugin, PluginField, EditorView } from '@codemirror/view';
import { EditorState, Facet, StateEffect, StateField, Compartment } from '@codemirror/state';
import { history, historyKeymap } from '@codemirror/history';
import { foldGutter, foldKeymap } from '@codemirror/fold';
import { indentOnInput, LanguageSupport } from '@codemirror/language';
import { lineNumbers } from '@codemirror/gutter';
import { defaultKeymap } from '@codemirror/commands';
import { bracketMatching } from '@codemirror/matchbrackets';
import { closeBrackets, closeBracketsKeymap } from '@codemirror/closebrackets';
import { highlightSelectionMatches, searchKeymap } from '@codemirror/search';
import { autocompletion, completionKeymap } from '@codemirror/autocomplete';
import { commentKeymap } from '@codemirror/comment';
import { rectangularSelection } from '@codemirror/rectangular-selection';
import { defaultHighlightStyle, HighlightStyle, tags } from '@codemirror/highlight';
import { lintKeymap, setDiagnostics } from '@codemirror/lint';
import React, { useEffect, useState, useRef, forwardRef, useImperativeHandle } from 'react';
import { Tree } from 'lezer';
import { dequal } from 'dequal';
import { javascript } from '@codemirror/lang-javascript';
import { json } from '@codemirror/lang-json';
import { StandardSQL, schemaCompletion, keywordCompletion } from '@codemirror/lang-sql';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

/// This is an extension value that just pulls together a whole lot of
/// extensions that you might want in a basic editor. It is meant as a
/// convenient helper to quickly set up CodeMirror without installing
/// and importing a lot of packages.
///
/// Specifically, it includes...
///
///  - [the default command bindings](#commands.defaultKeymap)
///  - [line numbers](#gutter.lineNumbers)
///  - [special character highlighting](#view.highlightSpecialChars)
///  - [the undo history](#history.history)
///  - [a fold gutter](#fold.foldGutter)
///  - [custom selection drawing](#view.drawSelection)
///  - [multiple selections](#state.EditorState^allowMultipleSelections)
///  - [reindentation on input](#language.indentOnInput)
///  - [the default highlight style](#highlight.defaultHighlightStyle) (as fallback)
///  - [bracket matching](#matchbrackets.bracketMatching)
///  - [bracket closing](#closebrackets.closeBrackets)
///  - [autocompletion](#autocomplete.autocompletion)
///  - [rectangular selection](#rectangular-selection.rectangularSelection)
///  - [active line highlighting](#view.highlightActiveLine)
///  - [selection match highlighting](#search.highlightSelectionMatches)
///  - [search](#search.searchKeymap)
///  - [commenting](#comment.commentKeymap)
///  - [linting](#lint.lintKeymap)
///
/// (You'll probably want to add some language package to your setup
/// too.)
///
/// This package does not allow customization. The idea is that, once
/// you decide you want to configure your editor more precisely, you
/// take this package's source (which is just a bunch of imports and
/// an array literal), copy it into your own code, and adjust it as
/// desired.
var basicSetup = [
    lineNumbers(),
    highlightSpecialChars(),
    history(),
    foldGutter(),
    drawSelection(),
    EditorState.allowMultipleSelections.of(true),
    indentOnInput(),
    defaultHighlightStyle.fallback,
    bracketMatching(),
    closeBrackets(),
    autocompletion(),
    rectangularSelection(),
    highlightActiveLine(),
    highlightSelectionMatches(),
    keymap.of(closeBracketsKeymap.concat( defaultKeymap,
        searchKeymap,
        historyKeymap,
        foldKeymap,
        commentKeymap,
        completionKeymap,
        lintKeymap )) ];

/**
 * hooks for mount callback
 * @param cb
 * @returns
 */
function useMount(cb) {
    useEffect(function () {
        cb && cb();
    }, []);
    return null;
}
/**
 * hook for unmount callback
 * @param cb
 * @returns
 */
function useUnmount(cb) {
    useEffect(function () {
        return function () {
            cb();
        };
    }, []);
    return null;
}

var panelConfig = Facet.define({
    combine: function combine(configs) {
        var topContainer, bottomContainer;
        for (var c of configs) {
            topContainer = topContainer || c.topContainer;
            bottomContainer = bottomContainer || c.bottomContainer;
        }
        return { topContainer: topContainer, bottomContainer: bottomContainer };
    }
});
var panelPlugin = ViewPlugin.fromClass(/*@__PURE__*/(function () {
    function anonymous(view) {
        this.input = view.state.facet(showPanel);
        this.specs = this.input.filter(function (s) { return s; });
        this.panels = this.specs.map(function (spec) { return spec(view); });
        var conf = view.state.facet(panelConfig);
        this.top = new PanelGroup(view, true, conf.topContainer);
        this.bottom = new PanelGroup(view, false, conf.bottomContainer);
        this.top.sync(this.panels.filter(function (p) { return p.top; }));
        this.bottom.sync(this.panels.filter(function (p) { return !p.top; }));
        for (var p of this.panels) {
            p.dom.classList.add("cm-panel");
            // FIXME drop on next breaking release
            if (p.class)
                { p.dom.classList.add(p.class); }
            if (p.mount)
                { p.mount(); }
        }
    }
    anonymous.prototype.update = function update (update$1) {
        var conf = update$1.state.facet(panelConfig);
        if (this.top.container != conf.topContainer) {
            this.top.sync([]);
            this.top = new PanelGroup(update$1.view, true, conf.topContainer);
        }
        if (this.bottom.container != conf.bottomContainer) {
            this.bottom.sync([]);
            this.bottom = new PanelGroup(update$1.view, false, conf.bottomContainer);
        }
        this.top.syncClasses();
        this.bottom.syncClasses();
        var input = update$1.state.facet(showPanel);
        if (input != this.input) {
            var specs = input.filter(function (x) { return x; });
            var panels = [], top = [], bottom = [], mount = [];
            for (var spec of specs) {
                var known = this.specs.indexOf(spec), panel = (void 0);
                if (known < 0) {
                    panel = spec(update$1.view);
                    mount.push(panel);
                }
                else {
                    panel = this.panels[known];
                    if (panel.update)
                        { panel.update(update$1); }
                }
                panels.push(panel);
                (panel.top ? top : bottom).push(panel);
            }
            this.specs = specs;
            this.panels = panels;
            this.top.sync(top);
            this.bottom.sync(bottom);
            for (var p of mount) {
                p.dom.classList.add("cm-panel");
                // FIXME drop on next breaking release
                if (p.class)
                    { p.dom.classList.add(p.class); }
                if (p.mount)
                    { p.mount(); }
            }
        }
        else {
            for (var p$1 of this.panels)
                if (p$1.update)
                    { p$1.update(update$1); }
        }
    };
    anonymous.prototype.destroy = function destroy () {
        this.top.sync([]);
        this.bottom.sync([]);
    };

    return anonymous;
}()), {
    provide: PluginField.scrollMargins.from(function (value) { return ({ top: value.top.scrollMargin(), bottom: value.bottom.scrollMargin() }); })
});
var PanelGroup = function PanelGroup(view, top, container) {
    this.view = view;
    this.top = top;
    this.container = container;
    this.dom = undefined;
    this.classes = "";
    this.panels = [];
    this.syncClasses();
};
PanelGroup.prototype.sync = function sync (panels) {
    this.panels = panels;
    this.syncDOM();
};
PanelGroup.prototype.syncDOM = function syncDOM () {
    if (this.panels.length == 0) {
        if (this.dom) {
            this.dom.remove();
            this.dom = undefined;
        }
        return;
    }
    if (!this.dom) {
        this.dom = document.createElement("div");
        this.dom.className = this.top ? "cm-panels cm-panels-top" : "cm-panels cm-panels-bottom";
        this.dom.style[this.top ? "top" : "bottom"] = "0";
        var parent = this.container || this.view.dom;
        parent.insertBefore(this.dom, this.top ? parent.firstChild : null);
    }
    var curDOM = this.dom.firstChild;
    for (var panel of this.panels) {
        if (panel.dom.parentNode == this.dom) {
            while (curDOM != panel.dom)
                { curDOM = rm(curDOM); }
            curDOM = curDOM.nextSibling;
        }
        else {
            this.dom.insertBefore(panel.dom, curDOM);
        }
    }
    while (curDOM)
        { curDOM = rm(curDOM); }
};
PanelGroup.prototype.scrollMargin = function scrollMargin () {
    return !this.dom || this.container ? 0
        : Math.max(0, this.top ? this.dom.getBoundingClientRect().bottom - this.view.scrollDOM.getBoundingClientRect().top
            : this.view.scrollDOM.getBoundingClientRect().bottom - this.dom.getBoundingClientRect().top);
};
PanelGroup.prototype.syncClasses = function syncClasses () {
    if (!this.container || this.classes == this.view.themeClasses)
        { return; }
    for (var cls of this.classes.split(" "))
        if (cls)
            { this.container.classList.remove(cls); }
    for (var cls$1 of (this.classes = this.view.themeClasses).split(" "))
        if (cls$1)
            { this.container.classList.add(cls$1); }
};
function rm(node) {
    var next = node.nextSibling;
    node.remove();
    return next;
}
var baseTheme = EditorView.baseTheme({
    ".cm-panels": {
        boxSizing: "border-box",
        position: "sticky",
        left: 0,
        right: 0
    },
    "&light .cm-panels": {
        backgroundColor: "#f5f5f5",
        color: "black"
    },
    "&light .cm-panels-top": {
        borderBottom: "1px solid #ddd"
    },
    "&light .cm-panels-bottom": {
        borderTop: "1px solid #ddd"
    },
    "&dark .cm-panels": {
        backgroundColor: "#333338",
        color: "white"
    }
});
/**
Opening a panel is done by providing a constructor function for
the panel through this facet. (The panel is closed again when its
constructor is no longer provided.) Values of `null` are ignored.
*/
var showPanel = Facet.define({
    enables: [panelPlugin, baseTheme]
});

var Viewbox = function Viewbox(parent) {
    this.node = document.createElement("div");
};
Viewbox.prototype.appendStyle = function appendStyle (cssStyle) {
    for (var prop in cssStyle) {
        this.node.style[prop] = cssStyle[prop];
    }
};
Viewbox.prototype.scroll = function scroll (scrollTop) {
    this.node.style.transform = "translateY(" + (Math.round(scrollTop)) + "px)";
};
Viewbox.prototype.hide = function hide () {
    this.node.style.visibility = "0";
};
Viewbox.prototype.show = function show () {
    this.node.style.visibility = "1";
};
Viewbox.prototype.resize = function resize (width, height) {
    this.node.style.width = String(width) + "px";
    this.node.style.height = String(height) + "px";
};
Viewbox.prototype.attach = function attach (parent) {
    parent.appendChild(this.node);
};
var CanvasElement = function CanvasElement(parent) {
    this.initLayer();
    this.attachCanvasLayer(parent);
};
CanvasElement.prototype.attachCanvasLayer = function attachCanvasLayer (parent) {
    parent.appendChild(this.node);
};
CanvasElement.prototype.initLayer = function initLayer () {
    this.node = document.createElement("canvas");
    this.ctx = this.node.getContext("2d");
    this.node.className = "cm-minimap-layer";
    this.node.style.zIndex = "0";
};
CanvasElement.prototype.resize = function resize (width, height) {
    this.node.style.width = String(width) + "px";
    this.node.style.height = String(height) + "px";
};
var MinimapElement = function MinimapElement(view) {
    this.view = view;
    this.node = document.createElement("div");
    this.node.className = "cm-minimap";
    this.view.dom.style.overflow = "hidden";
};
MinimapElement.prototype.resize = function resize (width, height) {
    if (width != this.width)
        { this.node.style.width = String(width) + "px"; }
    if (height != this.height)
        { this.node.style.height = String(height) + "px"; }
    this.width = width;
    this.height = height;
};
MinimapElement.prototype.setSide = function setSide (side) {
    this.view.dom.style.flexDirection =
        side === "left" ? "row-reverse" : "row";
};
MinimapElement.prototype.setBackground = function setBackground (bg) {
    this.node.style.background = bg;
};

var translateDiagnostics = function (input, view) {
    if ( input === void 0 ) input = {
    checkResult: { checkItems: [], rawSQL: "" },
};

    var checkItems = input.checkResult.checkItems; if ( checkItems === void 0 ) checkItems = [];
    var found = [];
    var maxLines = view.state.doc.lines;
    checkItems.forEach(function (item) {
        var error = item.error;
        var errorCode = item.errorCode;
        var lineOffsets = item.lineOffsets;
        lineOffsets.forEach(function (lineOffset) {
            var StartLine = lineOffset.StartLine;
            var EndLine = lineOffset.EndLine;
            var StartOffset = lineOffset.StartOffset;
            var EndOffset = lineOffset.EndOffset;
            var from, to;
            if (maxLines >= EndLine) {
                var ref = view.state.doc.line(StartLine + 1);
                var startFrom = ref.from;
                var startTo = ref.to;
                var ref$1 = view.state.doc.line(EndLine + 1);
                var endFrom = ref$1.from;
                var endTo = ref$1.to;
                if (startFrom === endFrom && startFrom === endTo) {
                    // same line
                    from = startFrom + StartOffset;
                    to = startTo + EndOffset;
                }
                else {
                    from = startFrom + StartOffset;
                    to = endFrom + EndOffset;
                }
                found.push({
                    from: from,
                    to: to,
                    severity: errorCode == -1 ? "error" : "warning",
                    message: error,
                });
            }
        });
    });
    return found.sort(function (a, b) {
        if (a.from < b.from) {
            return -1;
        }
        if (a.from > b.from) {
            return 1;
        }
        return 0;
    });
};
var isSameTextAccordingToDoc = function (editor, value) {
    if ( value === void 0 ) value = "";

    var doc = editor.state.doc.toJSON();
    var nextDocText = value.split(editor.state.lineBreak);
    if (doc.length !== nextDocText.length) {
        return false;
    }
    var i = doc.length - 1;
    while (i >= 0) {
        if (doc[i] !== nextDocText[i]) {
            return false;
        }
        i--;
    }
    return true;
};
function getStyle(element, attr) {
    return window.getComputedStyle
        ? window.getComputedStyle(element, null)[attr]
        : element.currentStyle[attr];
}

// const SpecialChars = [
//   "(",
//   ")",
//   "{",
//   "}",
//   "[",
//   "]",
//   ",",
//   ";",
//   "-",
//   "*",
// ]
var LineHeight = 1;
var WordGap = 5;
var Drawer = function Drawer(view, ctx, cache) {
    ///
    this.tree = Tree.empty;
    this.view = view;
    this.ctx = ctx;
    this.cache = cache;
};
/// 绘制内容
Drawer.prototype.draw = function draw (view) {
    var doc = view.state.doc;
    var totalLines = doc.lines;
    var textIter = doc.iter(1);
    while (!textIter.done) {
        console.log(textIter);
        textIter.next();
    }
    var index = 1;
    while (index <= totalLines) {
        this.drawLine(doc.line(index));
        index++;
    }
};
/// 处理一行的字符
Drawer.prototype.drawLine = function drawLine (line) {
        var this$1 = this;

    var text = line.text;
        var from = line.from;
    var offset = 2;
    if (/^\s.*/.test(text)) {
        offset = /\s+/.exec(text)[0].length + 10;
    }
    var tokens = text.split(/\s/);
    // Reset Prev Token
    this.posX = null;
    tokens.forEach(function (token, index) {
        var pos = from + line.text.indexOf(token) + 1;
        var color;
        var ref = this$1.view.domAtPos(pos);
            var node = ref.node;
        var cacheToken = this$1.cache.get(token + String(line.number));
        if (!cacheToken ||
            cacheToken.text !== token ||
            cacheToken.pos !== pos ||
            token !== "") {
            this$1.cache.set(token + String(line.number), {
                text: token,
                pos: pos,
            });
            color = getStyle(node.parentElement, "color");
            this$1.drawToken(line, {
                color: color,
                text: token,
            }, offset);
            // }
        }
    });
};
/// 处理单个字符
Drawer.prototype.drawToken = function drawToken (line, token, offset) {
    var color = token.color || "white";
    // let length: number, x: number, y: number
    var x = offset; // token渲染的左偏移
    var y = line.number + line.number * LineHeight; // token渲染的上偏移
    var length = token.text.trimStart().length * 5; // token的长度，原始长度太小，增加五倍的长度
    if (this.posX) {
        x = this.posX + WordGap;
    }
    this.drawRect(x, y, color, length, LineHeight);
    this.posX = x + length;
};
/// 绘制色块
Drawer.prototype.drawRect = function drawRect (x, y, color, width, height) {
    this.ctx.fillStyle = color;
    this.ctx.fillRect(x, y, width, height);
};
/// 清理canvas内容
Drawer.prototype.clear = function clear (x, y, w, h) {
    this.ctx.clearRect(x, y, w, h);
};

var TokenCache = function TokenCache() {
    this.tokens = new Map();
};
TokenCache.prototype.get = function get (key) {
    return this.tokens.get(key);
};
TokenCache.prototype.set = function set (key, value) {
    this.tokens.set(key, value);
};
TokenCache.prototype.clear = function clear () { };

var Minimap = function Minimap(view) {
    this.view = view;
    this.cache = new TokenCache();
    this.minimap = new MinimapElement(view);
    this.canvas = new CanvasElement(this.minimap.node);
    this.viewbox = new Viewbox(this.minimap.node);
    this.drawer = new Drawer(view, this.canvas.ctx, this.cache);
};
Minimap.prototype.resize = function resize (width, height) {
    this.minimap.resize(width, height);
    this.canvas.resize(width, height);
    this.viewbox.resize(width, height);
};
Minimap.prototype.setBackground = function setBackground (bg) {
    this.minimap.setBackground(bg);
};
// move .cm-minimap under .cm-editor
Minimap.prototype.detach = function detach () {
    var panels = this.view.dom.querySelector(".cm-panels");
    this.view.dom.removeChild(panels);
    return this;
};
Minimap.prototype.attach = function attach () {
    this.view.dom.appendChild(this.minimap.node);
    return this;
};
Minimap.prototype.render = function render (view) {
    if (!view) {
        view = this.view;
    }
    var cursorLine = view.state.doc.lineAt(view.state.selection.main.head);
    // clear canvas area below input line
    this.drawer.clear(0, cursorLine.number * 2, 80 * 5, view.dom.clientHeight);
    this.drawer.draw(view ? view : this.view);
};

var minimapConfig = Facet.define({
    combine: function combine(configs) {
        var minimap, minimapWidth, minimapSide;
        for (var config of configs) {
            minimap = config.minimap;
            minimapWidth = config.minimapWidth;
            minimapSide = config.minimapSide;
        }
        return {
            minimap: minimap,
            minimapSide: minimapSide || "left",
            minimapWidth: minimapWidth || 80,
        };
    },
});
var createMinimapPanel = function (view) {
    var config = view.state.facet(minimapConfig);
    var minimapPanel = new Minimap(view);
    return {
        top: true,
        dom: minimapPanel.minimap.node,
        update: function update(update$1) {
            if (update$1.docChanged) {
                minimapPanel.render(update$1.view);
            }
        },
        mount: function mount() {
            view.requestMeasure({
                write: function write() {
                    // FIXME 读取minimap是否是打开状态，是的话，才进行read方法里面的操作
                },
                read: function read(view) {
                    minimapPanel.resize(config.minimapWidth, view.dom.clientHeight);
                    minimapPanel.detach().attach();
                    minimapPanel.minimap.setSide(config.minimapSide);
                    minimapPanel.render();
                },
            });
        },
    };
};
var toggleMinimap = StateEffect.define();
var minimap = StateField.define({
    create: function create() {
        return true;
    },
    update: function update(update$1, tr) {
        for (var e of tr.effects)
            if (e.is(toggleMinimap))
                { update$1 = e.value; }
        return update$1;
    },
    provide: function provide(field) {
        return showPanel.from(field, function (show) { return show ? createMinimapPanel : null; });
    },
});
[
    minimap,
    EditorView.theme({
        ".cm-panel": {
            backgroundColor: "#21252b",
            color: "#abb2bf",
        },
    }) ];

function startFormat(view, configs) {
    var str = configs.parser(view.state.doc.toJSON().join(view.state.lineBreak));
    Promise.resolve(str).then(function (resolveStr) {
        view.dispatch({
            changes: [
                {
                    from: 0,
                    to: view.state.doc.length,
                    insert: resolveStr,
                } ],
        });
    });
}

/// feature flags
var MINIMAP_FLAG = false;

/// editable
var editableCompart = new Compartment();
function reconfigure$3(editable) {
    return editableCompart.reconfigure(EditorView.editable.of(editable));
}
function useEditableProp(value, view) {
    var ref = useState(function () { return editableCompart.of(EditorView.editable.of(value)); });
    var compart = ref[0];
    useEffect(function () {
        view &&
            view.dispatch({
                effects: [reconfigure$3(value)],
            });
    }, [value, view]);
    return compart;
}

function sql(config) {
    if ( config === void 0 ) config = {};

    var configs = Object.assign({}, config);
    if (!configs.schema) {
        configs.schema = {};
    }
    var lang = configs.dialect || StandardSQL;
    return new LanguageSupport(lang.language, [
        schemaCompletion(configs),
        keywordCompletion(lang, !!configs.upperCaseKeywords) ]);
}

/// language langOptions
var LANGUAGES = {
    javascript: javascript,
    sql: sql,
    json: json,
};
var languageCompart = new Compartment();
function reconfigure$2(language, langOptions) {
    return languageCompart.reconfigure(LANGUAGES[language](
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    //@ts-ignore
    langOptions));
}
function useLanguageProp(language, langOptions, view) {
    var prevLanguage = useRef({
        language: language,
        langOptions: langOptions,
    });
    var ref = useState(function () { return languageCompart.of(LANGUAGES[language](
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    //@ts-ignore
    langOptions)); });
    var compart = ref[0];
    useEffect(function () {
        if (view &&
            (prevLanguage.current.language !== language ||
                !dequal(langOptions, prevLanguage.current.langOptions))) {
            view.dispatch({
                effects: [reconfigure$2(language, langOptions)],
            });
        }
    }, [language, view, langOptions]);
    return compart;
}

// Using https://github.com/one-dark/vscode-one-dark-theme/ as reference for the colors
var chalky$1 = "#e5c07b", coral$1 = "#e06c75", cyan$1 = "#56b6c2", invalid$1 = "#ffffff", ivory$1 = "#abb2bf", stone$1 = "#7d8799", // Brightened compared to original to increase contrast
malibu$1 = "#61afef", sage$1 = "#98c379", whiskey$1 = "#d19a66", violet$1 = "#c678dd", darkBackground$1 = "#21252b", highlightBackground$1 = "#2c313a", background$1 = "#282c34", selection$1 = "#3E4451", cursor$1 = "#528bff";
/// The editor theme styles for One Dark.
var oneDarkTheme$1 = EditorView.theme({
    "&": {
        color: ivory$1,
        backgroundColor: background$1,
        "& ::selection": { backgroundColor: selection$1 },
        caretColor: cursor$1,
        height: "100%",
    },
    "&.cm-focused .cm-cursor": { borderLeftColor: cursor$1 },
    "&.cm-focused .cm-selectionBackground, .cm-selectionBackground": {
        backgroundColor: selection$1,
    },
    ".cm-panels": { backgroundColor: darkBackground$1, color: ivory$1 },
    ".cm-panels.cm-panels-top": { borderBottom: "2px solid black" },
    ".cm-panels.cm-panels-bottom": { borderTop: "2px solid black" },
    ".cm-searchMatch": {
        backgroundColor: "#72a1ff59",
        outline: "1px solid #457dff",
    },
    ".cm-searchMatch.cm-searchMatch-selected": {
        backgroundColor: "#6199ff2f",
    },
    ".cm-activeLine": { backgroundColor: highlightBackground$1 },
    ".cm-selectionMatch": { backgroundColor: "#aafe661a" },
    ".cm-matchingBracket, .cm-nonmatchingBracket": {
        backgroundColor: "#bad0f847",
        outline: "1px solid #515a6b",
    },
    ".cm-gutters": {
        backgroundColor: background$1,
        color: stone$1,
        border: "none",
    },
    ".cm-lineNumbers .cm-gutterElement": { color: "inherit" },
    ".cm-foldPlaceholder": {
        backgroundColor: "transparent",
        border: "none",
        color: "#ddd",
    },
    ".cm-tooltip": {
        border: "1px solid #181a1f",
        backgroundColor: darkBackground$1,
    },
    ".cm-tooltip-autocomplete": {
        "& > ul > li[aria-selected]": {
            backgroundColor: highlightBackground$1,
            color: ivory$1,
        },
        "& > ul > li > cm-completionIcon": {
            paddingRight: "2em",
        },
    },
}, { dark: true });
/// The highlighting style for code in the One Dark theme.
var oneDarkHighlightStyle$1 = HighlightStyle.define([
    { tag: tags.keyword, color: violet$1 },
    {
        tag: [
            tags.name,
            tags.deleted,
            tags.character,
            tags.propertyName,
            tags.macroName ],
        color: coral$1,
    },
    {
        tag: [tags.function(tags.variableName), tags.labelName],
        color: malibu$1,
    },
    {
        tag: [
            tags.color,
            tags.constant(tags.name),
            tags.standard(tags.name) ],
        color: whiskey$1,
    },
    { tag: [tags.definition(tags.name), tags.separator], color: ivory$1 },
    {
        tag: [
            tags.typeName,
            tags.className,
            tags.number,
            tags.changed,
            tags.annotation,
            tags.modifier,
            tags.self,
            tags.namespace ],
        color: chalky$1,
    },
    {
        tag: [
            tags.operator,
            tags.operatorKeyword,
            tags.url,
            tags.escape,
            tags.regexp,
            tags.link,
            tags.special(tags.string) ],
        color: cyan$1,
    },
    { tag: [tags.meta, tags.comment], color: stone$1 },
    { tag: tags.strong, fontWeight: "bold" },
    { tag: tags.emphasis, fontStyle: "italic" },
    { tag: tags.link, color: stone$1, textDecoration: "underline" },
    { tag: tags.heading, fontWeight: "bold", color: coral$1 },
    {
        tag: [tags.atom, tags.bool, tags.special(tags.variableName)],
        color: whiskey$1,
    },
    {
        tag: [tags.processingInstruction, tags.string, tags.inserted],
        color: sage$1,
    },
    { tag: tags.invalid, color: invalid$1 } ]);
/// Extension to enable the One Dark theme (both the editor theme and
/// the highlight style).
var oneDark$1 = [oneDarkTheme$1, oneDarkHighlightStyle$1];

// Using https://github.com/one-dark/vscode-one-dark-theme/ as reference for the colors
var chalky = "#e5c07b", coral = "#e06c75", cyan = "#56b6c2", invalid = "#ffffff", ivory = "#abb2bf", stone = "#7d8799", // Brightened compared to original to increase contrast
malibu = "#61afef", sage = "#98c379", whiskey = "#d19a66", violet = "#c678dd", darkBackground = "#21252b", highlightBackground = "#e2e2e2", background = "#ffffff", selection = "#3E4451", cursor = "#528bff";
/// The editor theme styles for One Dark.
var oneDarkTheme = EditorView.theme({
    "&": {
        color: ivory,
        backgroundColor: background,
        "& ::selection": { backgroundColor: selection },
        caretColor: cursor,
        height: '100%'
    },
    "&.cm-focused .cm-cursor": { borderLeftColor: cursor },
    "&.cm-focused .cm-selectionBackground, .cm-selectionBackground": {
        backgroundColor: selection,
    },
    ".cm-panels": { backgroundColor: darkBackground, color: ivory },
    ".cm-panels.cm-panels-top": { borderBottom: "2px solid black" },
    ".cm-panels.cm-panels-bottom": { borderTop: "2px solid black" },
    ".cm-searchMatch": {
        backgroundColor: "#72a1ff59",
        outline: "1px solid #457dff",
    },
    ".cm-searchMatch.cm-searchMatch-selected": {
        backgroundColor: "#6199ff2f",
    },
    ".cm-activeLine": { backgroundColor: highlightBackground },
    ".cm-selectionMatch": { backgroundColor: "#aafe661a" },
    ".cm-matchingBracket, .cm-nonmatchingBracket": {
        backgroundColor: "#bad0f847",
        outline: "1px solid #515a6b",
    },
    ".cm-gutters": {
        backgroundColor: background,
        color: stone,
        border: "none",
    },
    ".cm-lineNumbers .cm-gutterElement": { color: "inherit" },
    ".cm-foldPlaceholder": {
        backgroundColor: "transparent",
        border: "none",
        color: "#ddd",
    },
    ".cm-tooltip": {
        border: "1px solid #181a1f",
        backgroundColor: darkBackground,
    },
    ".cm-tooltip-autocomplete": {
        "& > ul > li[aria-selected]": {
            backgroundColor: highlightBackground,
            color: ivory,
        },
        "& > ul > li > cm-completionIcon": {
            paddingRight: "2em",
        },
    },
}, { dark: true });
/// The highlighting style for code in the One Dark theme.
var oneDarkHighlightStyle = HighlightStyle.define([
    { tag: tags.keyword, color: violet },
    {
        tag: [
            tags.name,
            tags.deleted,
            tags.character,
            tags.propertyName,
            tags.macroName ],
        color: coral,
    },
    {
        tag: [tags.function(tags.variableName), tags.labelName],
        color: malibu,
    },
    {
        tag: [
            tags.color,
            tags.constant(tags.name),
            tags.standard(tags.name) ],
        color: whiskey,
    },
    { tag: [tags.definition(tags.name), tags.separator], color: ivory },
    {
        tag: [
            tags.typeName,
            tags.className,
            tags.number,
            tags.changed,
            tags.annotation,
            tags.modifier,
            tags.self,
            tags.namespace ],
        color: chalky,
    },
    {
        tag: [
            tags.operator,
            tags.operatorKeyword,
            tags.url,
            tags.escape,
            tags.regexp,
            tags.link,
            tags.special(tags.string) ],
        color: cyan,
    },
    { tag: [tags.meta, tags.comment], color: stone },
    { tag: tags.strong, fontWeight: "bold" },
    { tag: tags.emphasis, fontStyle: "italic" },
    { tag: tags.link, color: stone, textDecoration: "underline" },
    { tag: tags.heading, fontWeight: "bold", color: coral },
    {
        tag: [tags.atom, tags.bool, tags.special(tags.variableName)],
        color: whiskey,
    },
    {
        tag: [tags.processingInstruction, tags.string, tags.inserted],
        color: sage,
    },
    { tag: tags.invalid, color: invalid } ]);
/// Extension to enable the One Dark theme (both the editor theme and
/// the highlight style).
var oneDark = [oneDarkTheme, oneDarkHighlightStyle];

/// theme
var themeCompart = new Compartment();
var THEMES = {
    dark: oneDark$1,
    light: oneDark,
};
function reconfigure$1(theme) {
    return themeCompart.reconfigure(THEMES[theme]);
}
function useThemeProp(theme, view) {
    var ref = useState(function () { return themeCompart.of(THEMES[theme]); });
    var compart = ref[0];
    useEffect(function () {
        view &&
            view.dispatch({
                effects: [reconfigure$1(theme)],
            });
    }, [theme, view]);
    return compart;
}

function useDiagnostics(diagnostics, view) {
    useEffect(function () {
        if (view) {
            var founds = translateDiagnostics(diagnostics, view);
            var first = founds[0];
            // scroll into this first diagnostric result
            if (first) {
                view.scrollPosIntoView(first.from);
            }
            view.dispatch(setDiagnostics(view.state, translateDiagnostics(diagnostics, view)));
        }
    }, [diagnostics, view]);
}

/// editable
var extensionsCompart = new Compartment();
function reconfigure(extensions) {
    return extensionsCompart.reconfigure(extensions);
}
function useExtensionsProp(extensions, view) {
    var ref = useState(function () { return extensionsCompart.of(extensions); });
    var compart = ref[0];
    useEffect(function () {
        view &&
            view.dispatch({
                effects: [reconfigure(extensions)],
            });
    }, [extensions, view]);
    return compart;
}

function useChangedValue(value, view) {
    useEffect(function () {
        if (view && !isSameTextAccordingToDoc(view, value)) {
            view.dispatch({
                changes: [
                    {
                        from: 0,
                        to: view.state.doc.length,
                        insert: value,
                    } ],
            });
        }
    }, [value, view]);
}
function listenValueChangeAndInvokeCallback(handler) {
    return EditorView.updateListener.of(function (ref) {
        var docChanged = ref.docChanged;
        var state = ref.state;

        var editable = editableCompart.get(state);
        if (docChanged &&
            handler &&
            editable &&
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            state.facet(editable.facet)) {
            handler(state.doc.toJSON().join(state.lineBreak));
        }
    });
}

function ReactCodemirror(props, ref) {
    var onChange = props.onChange;
    var others = __rest(props, ["onChange"]);
    var element = useRef();
    var editor = useRef();
    useImperativeHandle(
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    ref, function () { return ({
        format: function format(configs) {
            startFormat(editor.current, configs);
        },
    }); }, [editor]);
    var themeCompart = useThemeProp(props.theme || "dark", editor.current);
    var editCompart = useEditableProp(props.editable != undefined ? props.editable : true, editor.current);
    var languageCompart = useLanguageProp(props.language, props.langOptions, editor.current);
    var extensionsCompart = useExtensionsProp(props.extensions || [], editor.current);
    useDiagnostics(props.diagnostics, editor.current);
    useChangedValue(props.value, editor.current);
    useMount(function () {
        var state = EditorState.create({
            doc: props.defaultValue || props.value || "",
            extensions: [
                basicSetup,
                editCompart,
                themeCompart,
                languageCompart,
                extensionsCompart,
                listenValueChangeAndInvokeCallback(onChange),
                MINIMAP_FLAG  ].filter(Boolean),
        });
        editor.current = new EditorView({
            state: state,
            parent: element.current,
        });
    });
    useUnmount(function () { return editor.current.destroy(); });
    return (React.createElement( 'div', Object.assign({}, { ref: element, className: "codemirror-editor-body" }, others)));
}
var ReactCodemirror$1 = forwardRef(ReactCodemirror);

export { ReactCodemirror$1 as ReactCodemirror };
